// 
// Copyright (C) 2006, NinjaTrader LLC <ninjatrader@ninjatrader.com>.
// NinjaTrader reserves the right to modify or overwrite this NinjaScript component with each release.
//

#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
	/// <summary>
	/// Keltner Channel. The Keltner Channel is a similar indicator to Bollinger Bands. Here the midline is a standard moving average with the upper and lower bands offset by the SMA of the difference between the high and low of the previous bars. The offset multiplier as well as the SMA period is configurable.
	/// </summary>
	[Description("The Keltner Channel is a similar indicator to Bollinger Bands. Here the midline is a standard moving average with the upper and lower bands offset by the SMA of the difference between the high and low of the previous bars. The offset multiplier as well as the SMA period is configurable.")]
	[Gui.Design.DisplayName("Keltner Channel with Color")]
	public class KeltnerChannelwColor : Indicator
	{
		#region Variables
		private	int					period				= 8;
		private double				offsetMultiplier	= 1.3;
		private DataSeries		diff;
		private Color bandAreaColor = Color.Blue;
		private int bandAreaColorOpacity = 2;
		#endregion

		/// <summary>
		/// This method is used to configure the indicator and is called once before any bar data is loaded.
		/// </summary>
		protected override void Initialize()
		{
			Add(new Plot(Color.Blue,     "Upper"));
			Add(new Plot(Color.Blue,     "Lower"));
			Add(new Plot(Color.DarkGray, "Midline"));
			

			diff				= new DataSeries(this);

			Overlay				= true;
			PriceTypeSupported	= false;
		}

		/// <summary>
		/// Called on each bar update event (incoming tick).
		/// </summary>
		protected override void OnBarUpdate()
		{
			diff.Set(High[0] - Low[0]);

			double middle	= SMA(Typical, Period)[0];
			double offset	= SMA(diff, Period)[0] * offsetMultiplier;

			double upper	= middle + offset;
			double lower	= middle - offset;
			
			Upper.Set(upper);
			Lower.Set(lower);
			Midline.Set(middle);
						
		}

		#region Properties
		/// <summary>
		/// </summary>
		[Description("Numbers of bars used for calculations")]
		[Category("Parameters")]
		public int Period
		{
			get { return period; }
			set { period = Math.Max(1, value); }
		}

		/// <summary>
		/// </summary>
		[Description("How much to expand the upper and lower band from the normal offset")]
		[Category("Parameters")]
		[Gui.Design.DisplayNameAttribute("Offset multiplier")]
		public double OffsetMultiplier
		{
			get { return offsetMultiplier; }
			set { offsetMultiplier = Math.Max(1, value); }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries Midline
		{
			get { return Values[2]; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries Upper
		{
			get { return Values[0]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries Lower
		{
			get { return Values[1]; }
		}
		
		[XmlIgnore()]
		[Description("Band Color")]
		[Category("Visual")]
		[Gui.Design.DisplayNameAttribute("Band Color")]
		public Color BandAreaColor
		{
			get { return bandAreaColor; }
			set { bandAreaColor = value; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		public string BandAreaColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(bandAreaColor); }
			set { bandAreaColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		[XmlIgnore()]
		[Description("Band Color Opacity")]
		[Category("Visual")]
		[Gui.Design.DisplayNameAttribute("Band Color Opacity")]
		public int BandAreaColorOpacity
		{
			get { return bandAreaColorOpacity; }
			set { bandAreaColorOpacity = Math.Max(value, 1); }
		}
        #endregion
		
		public override void Plot(Graphics graphics, Rectangle bounds, double min, double max)
		{
			
			base.Plot(graphics, bounds, min, max);

			if (Bars == null || /*Plots.Count < 2 ||*/ ChartControl == null)
				return;
				
			//SolidBrush brush = new SolidBrush(BandAreaColor);
			SolidBrush brush = new SolidBrush(Color.FromArgb(bandAreaColorOpacity*20,bandAreaColor));
			int	barWidth = ChartControl.ChartStyle.GetBarPaintWidth(ChartControl.BarWidth);
			SmoothingMode oldSmoothingMode = graphics.SmoothingMode;
			GraphicsPath path = new GraphicsPath();

			for (int seriesCount = 0; seriesCount < 2; seriesCount++)
			{
				int	lastX = -1;
				int	lastY = -1;
				DataSeries	series = (DataSeries) Values[seriesCount];
				double val = 0;
				Gui.Chart.Plot plot = Plots[seriesCount];

				for (int count = 0; count < ChartControl.BarsPainted; count++)
				{
					int idx = ChartControl.LastBarPainted - ChartControl.BarsPainted + 1 + count;
					if (idx < 0 || idx >= Input.Count || (!ChartControl.ShowBarsRequired && idx < BarsRequired))
						continue;

					val	= series.Get(idx);
					if (val == 0)
						continue;

					int			x	= (int) (ChartControl.CanvasRight - ChartControl.BarMarginRight - barWidth / 2
						- (ChartControl.BarsPainted - 1) * ChartControl.BarSpace + count * ChartControl.BarSpace) + 1;
					int			y	= (int) ((bounds.Y + bounds.Height) - ((val - min ) / Gui.Chart.ChartControl.MaxMinusMin(max, min)) * bounds.Height);

					if (lastX >= 0) 
					{
						path.AddLine(lastX - plot.Pen.Width / 2, lastY, x - plot.Pen.Width / 2, y);
					}

					lastX	= x;
					lastY	= y;
				}
				path.Reverse();
				
			}
			graphics.SmoothingMode = SmoothingMode.AntiAlias;
			//graphics.DrawPath(new Pen(Color.Orange), path1);
			graphics.FillPath(brush, path);
			graphics.SmoothingMode = oldSmoothingMode;
		}
		
		
		
		
	}
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private KeltnerChannelwColor[] cacheKeltnerChannelwColor = null;

        private static KeltnerChannelwColor checkKeltnerChannelwColor = new KeltnerChannelwColor();

        /// <summary>
        /// The Keltner Channel is a similar indicator to Bollinger Bands. Here the midline is a standard moving average with the upper and lower bands offset by the SMA of the difference between the high and low of the previous bars. The offset multiplier as well as the SMA period is configurable.
        /// </summary>
        /// <returns></returns>
        public KeltnerChannelwColor KeltnerChannelwColor(double offsetMultiplier, int period)
        {
            return KeltnerChannelwColor(Input, offsetMultiplier, period);
        }

        /// <summary>
        /// The Keltner Channel is a similar indicator to Bollinger Bands. Here the midline is a standard moving average with the upper and lower bands offset by the SMA of the difference between the high and low of the previous bars. The offset multiplier as well as the SMA period is configurable.
        /// </summary>
        /// <returns></returns>
        public KeltnerChannelwColor KeltnerChannelwColor(Data.IDataSeries input, double offsetMultiplier, int period)
        {
            checkKeltnerChannelwColor.OffsetMultiplier = offsetMultiplier;
            offsetMultiplier = checkKeltnerChannelwColor.OffsetMultiplier;
            checkKeltnerChannelwColor.Period = period;
            period = checkKeltnerChannelwColor.Period;

            if (cacheKeltnerChannelwColor != null)
                for (int idx = 0; idx < cacheKeltnerChannelwColor.Length; idx++)
                    if (Math.Abs(cacheKeltnerChannelwColor[idx].OffsetMultiplier - offsetMultiplier) <= double.Epsilon && cacheKeltnerChannelwColor[idx].Period == period && cacheKeltnerChannelwColor[idx].EqualsInput(input))
                        return cacheKeltnerChannelwColor[idx];

            KeltnerChannelwColor indicator = new KeltnerChannelwColor();
            indicator.SetUp();
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.OffsetMultiplier = offsetMultiplier;
            indicator.Period = period;

            KeltnerChannelwColor[] tmp = new KeltnerChannelwColor[cacheKeltnerChannelwColor == null ? 1 : cacheKeltnerChannelwColor.Length + 1];
            if (cacheKeltnerChannelwColor != null)
                cacheKeltnerChannelwColor.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheKeltnerChannelwColor = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// The Keltner Channel is a similar indicator to Bollinger Bands. Here the midline is a standard moving average with the upper and lower bands offset by the SMA of the difference between the high and low of the previous bars. The offset multiplier as well as the SMA period is configurable.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.KeltnerChannelwColor KeltnerChannelwColor(double offsetMultiplier, int period)
        {
            return _indicator.KeltnerChannelwColor(Input, offsetMultiplier, period);
        }

        /// <summary>
        /// The Keltner Channel is a similar indicator to Bollinger Bands. Here the midline is a standard moving average with the upper and lower bands offset by the SMA of the difference between the high and low of the previous bars. The offset multiplier as well as the SMA period is configurable.
        /// </summary>
        /// <returns></returns>
        public Indicator.KeltnerChannelwColor KeltnerChannelwColor(Data.IDataSeries input, double offsetMultiplier, int period)
        {
            return _indicator.KeltnerChannelwColor(input, offsetMultiplier, period);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// The Keltner Channel is a similar indicator to Bollinger Bands. Here the midline is a standard moving average with the upper and lower bands offset by the SMA of the difference between the high and low of the previous bars. The offset multiplier as well as the SMA period is configurable.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.KeltnerChannelwColor KeltnerChannelwColor(double offsetMultiplier, int period)
        {
            return _indicator.KeltnerChannelwColor(Input, offsetMultiplier, period);
        }

        /// <summary>
        /// The Keltner Channel is a similar indicator to Bollinger Bands. Here the midline is a standard moving average with the upper and lower bands offset by the SMA of the difference between the high and low of the previous bars. The offset multiplier as well as the SMA period is configurable.
        /// </summary>
        /// <returns></returns>
        public Indicator.KeltnerChannelwColor KeltnerChannelwColor(Data.IDataSeries input, double offsetMultiplier, int period)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.KeltnerChannelwColor(input, offsetMultiplier, period);
        }

    }
}
#endregion
